#!/usr/bin/python

# Copyright 2013-2014 Canonical Ltd. All rights reserved.
# Author: Haw Loeung <haw.loeung@canonical.com>
# Author: Caio Begotti <caio.begotti@canonical.com>

import apt
import json
import os
import pwd
import random
import subprocess
import sys
import time

juju_warning_header = """#
#    "             "
#  mmm   m   m   mmm   m   m
#    #   #   #     #   #   #
#    #   #   #     #   #   #
#    #   "mm"#     #   "mm"#
#    #             #
#  ""            ""
# This file is managed by Juju. Do not make local changes.
#"""


def log(debug, msg):
    if debug:
        print "DEBUG: %s: %s" % (os.environ["JUJU_UNIT_NAME"], msg)
        sys.stdout.flush()


def run(command, *args, **kwargs):
    try:
        output = subprocess.check_output(command, *args, **kwargs)
        return output
    except Exception, e:
        print str(e)
        raise


def config_get(scope=None):
    try:
        config_cmd_line = ["config-get", "--format=json"]
        if scope is not None:
            config_cmd_line.append(scope)
        config_data = json.loads(run(config_cmd_line))
    except Exception:
        config_data = None
    finally:
        # Debug option isn't available until now.
        log(config_data["debug"], "Retrieving config: scope=%s" % scope)
        log(config_data["debug"], config_data)
        return config_data


def package_install(debug, packages):
    cache = apt.Cache()

    # As per LP: #1068624, subordinates can fire at the same time potentially
    # causing apt to fail if another subordinate is still installing packages.
    # Let's loop and wait till the other one is done.
    for i in range(30):
        try:
            cache.update()
        except Exception, e:
            log(debug, str(e))
            time.sleep(random.randint(2, 10))
            continue
        break

    cache.open(None)
    with cache.actiongroup():
        for pkg in packages:
            p = cache[pkg]
            if p.is_installed:
                log(debug, "Package already installed: %s" % pkg)
            else:
                log(debug, "Installing package: %s" % pkg)
                p.mark_install()

    # As per LP: #1068624, subordinates can fire at the same time potentially
    # causing apt to fail if another subordinate is still installing packages.
    # Let's loop and wait till the other one is done.
    for i in range(30):
        try:
            cache.commit()
        except Exception, e:
            log(debug, str(e))
            time.sleep(random.randint(2, 10))
            continue
        break

    return 0


def a2enmod(module=None):
    if module is not None:
        config_cmd_line = ["a2enmod", module]
        return run(config_cmd_line)


def a2dismod(module=None):
    if module is not None:
        config_cmd_line = ["a2dismod", module]
        return run(config_cmd_line)


def hook_install(config):
    package_install(config["debug"], ["libapache2-mod-python",
                                      "python-apache-openid"])
    a2enmod("python")
    return 0


def hook_config_changed(config):
    providerspath = config["openid_store"] + '/providers.txt'
    teamspath = config["openid_store"] + '/teams.txt'
    userspath = config["openid_store"] + '/users.txt'
    dirpath = os.path.dirname(providerspath)

    # storage directory setup
    if not os.path.exists(dirpath):
        os.makedirs(dirpath)

    apache_uid = pwd.getpwnam('www-data').pw_uid
    apache_gid = pwd.getpwnam('www-data').pw_gid

    try:
        os.chown(dirpath, apache_uid, apache_gid)
    except Exception, e:
        print str(e)
        return 1

    # saves the ops files
    try:
        fh = open(providerspath, "w")
        for line in config["allowed_providers"].split():
            fh.write("%s\n" % line)
    except Exception, e:
        print str(e)
        return 1
    else:
        fh.close()

    # then saves a list of authorized teams
    try:
        fh = open(teamspath, "w")
        for line in config["authorized_teams"].split():
            fh.write("%s\n" % line)
    except Exception, e:
        print str(e)
        return 1
    else:
        fh.close()

    # or/and saves a list of authorized users
    try:
        fh = open(userspath, "w")
        for line in config["authorized_users"].split():
            fh.write("%s\n" % line)
    except Exception, e:
        print str(e)
        return 1
    else:
        fh.close()



    run(["service", "apache2", "reload"])
    return 0


def hook_start():
    run(["service", "apache2", "reload"])
    return 0


def hook_stop():
    a2dismod("python")
    run(["service", "apache2", "reload"])
    return 0


def main():
    hook_name = os.path.basename(sys.argv[0])
    config = config_get()
    config["juju_warning_header"] = juju_warning_header

    log(config["debug"], "Hook: %s" % hook_name)
    if hook_name == "install":
        sys.exit(hook_install(config))
    elif hook_name == "config-changed":
        sys.exit(hook_config_changed(config))
    elif hook_name == "upgrade-charm":
        ret = hook_install(config)
        if ret:
            sys.exit(ret)
        sys.exit(hook_config_changed(config))
    elif hook_name == "start":
        sys.exit(hook_start())
    elif hook_name == "stop":
        sys.exit(hook_stop())
    else:
        print "Unsupported hook: %s" % hook_name
        sys.exit(0)


if __name__ == "__main__":
    main()
